/*********************************************************************/
/*                                                                   */
/*    killAllChannels2.c                                             */
/*    Set to 0 all channels	                                     */ 
/*    Return OK or any failure message                               */
/*								     */
/*    Works on more Caenet PCI Controller 			     */
/*                                                                   */
/*    From HSCAENETLib-1-4-extended/sy546ctrl.c                      */
/*    Luca Boni, 11 settembre 2003                                   */
/*                                                                   */
/*    This program is free software: you can redistribute it	     */
/*    and/or modify it under the terms of the GNU General Public     */
/*    License version 2 as published by the Free Software Foundation */
/*                                                                   */
/*********************************************************************/

#include  <stdio.h>
#include  <string.h>
#include  <stdlib.h>
#include  <errno.h>

#include  "HSCAENETLib.h"

/*******************************************************/
/*            internal & external constants            */     
/*******************************************************/

// Some of the Caenet Codes [word type]: -> see TECHNICAL INFORMATION MANUAL MOD.SY 546,
// MASTER-TO-SLAVE DATA PACKET DESCRIPTION pag.60
#define   KILL_ALL_CHANNELS                     0x35
#define   CONFIRM_KILL_CHANNELS                 0x36

// Globals
static int     response, HSCAENETDev, code, a1303index;

/*******************************************************/
/*                  functions                          */ 
/*******************************************************/

void initCard (void);
void endCard (void);
static char *DecodeCAENETResp (int resp);


int main (int argc, char *argv[]) {
 
 int crateNum;

 if (argc!=3) {
    printf("\nUsage:  killAllChannels2 caenetNum crateNum[1-8]\n\n");
    exit(-1);
 }

 a1303index = atoi(argv[1]);
 crateNum = atoi(argv[2]);

 initCard();

 code = KILL_ALL_CHANNELS;
 if ((response=HSCAENETComm(HSCAENETDev, code, crateNum, (NULL), (0), (NULL))) != TUTTOK) {
          printf("HSCAENETComm channels kill failed. %s\n", DecodeCAENETResp(response));
	  endCard();
	  exit(-1);
  } else {
     code = CONFIRM_KILL_CHANNELS;
     if ((response=HSCAENETComm(HSCAENETDev, code, crateNum, (NULL), (0), (NULL))) != TUTTOK) {
          printf("HSCAENETComm channels kill failed. %s\n", DecodeCAENETResp(response));
	  endCard();
	  exit(-1);
     }
  }

  puts("OK");
  endCard();
  return EXIT_SUCCESS;

}

/*******************************************************/
/*                   Subroutine initCard               */
/*  : connecting A1303                                 */  
/*******************************************************/

void initCard (void) {  

  long   timeout = 100;  // that is 1 sec

  HSCAENETDev = HSCAENETCardInit("A1303", &a1303index);
  if ( HSCAENETDev == -1 ) {
     puts("HSCAENETCardInit failed");
     exit(-1);
  } 
  if ( ( response = HSCAENETCardReset(HSCAENETDev) ) != TUTTOK ) {
        printf("HSCAENETCardReset failed. %s\n", DecodeCAENETResp(response));
	endCard();
	exit(-1);
  }
  if ( ( response = HSCAENETTimeout(HSCAENETDev, timeout) ) != TUTTOK ) {
        printf("HSCAENETTimeout failed. %s\n", DecodeCAENETResp(response));
	endCard();
	exit(-1);
  }

}

/*******************************************************/
/*                   Subroutine endCard                */
/*  : disconnecting A1303                              */
/*******************************************************/

void endCard (void) { HSCAENETCardEnd(HSCAENETDev); }

/*******************************************************/
/*             Subroutine DecodeCAENETResp             */
/*  : decode response from CAENET                      */     
/*  : return error response                            */
/*******************************************************/
static char *DecodeCAENETResp (int resp) {

  static char buf[80];

  if ( resp == E_OS )
     strcpy(buf, strerror(errno));
  else
     switch( resp ) {
        case E_LESS_DATA:
           strcpy(buf, "CaeNet Error: Less data received" );
           break;
        case E_TIMEOUT:
           strcpy(buf, "CaeNet Error: Timeout expired" );
           break;
        case E_NO_DEVICE:
           strcpy(buf, "CaeNet Error: High Speed CAENET device not found" );
           break;
        case E_A303_BUSY:
           sprintf(buf, "Another instance of a303lib.dll already loaded" );
           break;
        default:
           sprintf(buf, "CaeNet Error: %d", resp );
           break;
     }

  return buf;
}
