/*********************************************************************/
/*                                                                   */
/*    switchChannels.c                                               */
/*                                                                   */
/*    Switch ON/OFF a channel (Pw = 1/0)                             */  
/*    Return OK or any failure message                               */
/*                                                                   */
/*    From HSCAENETLib-1-4-extended/sy546ctrl.c                      */
/*    Luca Boni, 11 settembre 2003                                   */
/*                                                                   */
/*    This program is free software: you can redistribute it	     */
/*    and/or modify it under the terms of the GNU General Public     */
/*    License version 2 as published by the Free Software Foundation */
/*                                                                   */
/*********************************************************************/

#include  <stdio.h>
#include  <string.h>
#include  <stdlib.h>
#include  <errno.h>
#include  <sys/types.h>
#include  <unistd.h>

#include  "HSCAENETLib.h"

// Globals
static int	response, HSCAENETDev, code;

/*******************************************************/
/*                  functions                          */ 
/*******************************************************/

void initCard (void);
void endCard (void);
static char *DecodeCAENETResp (int resp);


int main(int argc, char *argv[]) {

  int crateNum, board, channel, turn, NUMCHANNELS=12;
  unsigned short    flagMask[1];

  if (argc != 4) {
    printf("\nUsage:  switchChannels crateNum[1-8] board[0-7] ON/OFF[1-0]\n\n");
    exit(-1);
  }

  initCard();
  
  crateNum =  atoi(argv[1]);
  board = atoi(argv[2]);
  channel = board * NUMCHANNELS;
  turn = atoi(argv[3]);

  code = channel * 256 + 24;	  // ((bd*12) + ch) * 256 + 24  => %n18

  if (turn == 1) {
     flagMask[0] = 55496;
     if ((response=HSCAENETComm(HSCAENETDev, code, crateNum, (flagMask), (sizeof(flagMask)), (NULL))) != TUTTOK) {
          printf("HSCAENETComm switch On failed. %s\n", DecodeCAENETResp(response));
	  endCard();
	  exit(-1);
     }
  } else if (turn == 0) {
     flagMask[0] = 55296;
     if ((response=HSCAENETComm(HSCAENETDev, code, crateNum, (flagMask), (sizeof(flagMask)), (NULL))) != TUTTOK) {
          printf("HSCAENETComm switch Off failed. %s\n", DecodeCAENETResp(response));
	  endCard();
	  exit(-1);
     }
  }

  puts("OK");
  endCard();
  return EXIT_SUCCESS;

}

/*******************************************************/
/*                   Subroutine initCard               */
/*  : connecting A1303                                 */  
/*******************************************************/

void initCard (void) {  
  
  int    a1303index = 0;
  long   timeout = 100;  // that is 1 sec

  HSCAENETDev = HSCAENETCardInit("A1303", &a1303index);
  if ( HSCAENETDev == -1 ) {
     puts("HSCAENETCardInit failed");
     exit(-1);
  } 
  if ( ( response = HSCAENETCardReset(HSCAENETDev) ) != TUTTOK ) {
        printf("HSCAENETCardReset failed. %s\n", DecodeCAENETResp(response));
	endCard();
	exit(-1);
  }
  if ( ( response = HSCAENETTimeout(HSCAENETDev, timeout) ) != TUTTOK ) {
        printf("HSCAENETTimeout failed. %s\n", DecodeCAENETResp(response));
	endCard();
	exit(-1);
  }

}

/*******************************************************/
/*                   Subroutine endCard                */
/*  : disconnecting A1303                              */
/*******************************************************/

void endCard (void) { HSCAENETCardEnd(HSCAENETDev); }

/*******************************************************/
/*             Subroutine DecodeCAENETResp             */
/*  : decode response from CAENET                      */     
/*  : return error response                            */
/*******************************************************/
static char *DecodeCAENETResp (int resp) {

  static char buf[80];

  if ( resp == E_OS )
     strcpy(buf, strerror(errno));
  else
     switch( resp ) {
        case E_LESS_DATA:
           strcpy(buf, "CaeNet Error: Less data received" );
           break;
        case E_TIMEOUT:
           strcpy(buf, "CaeNet Error: Timeout expired" );
           break;
        case E_NO_DEVICE:
           strcpy(buf, "CaeNet Error: High Speed CAENET device not found" );
           break;
        case E_A303_BUSY:
           sprintf(buf, "Another instance of a303lib.dll already loaded" );
           break;
        default:
           sprintf(buf, "CaeNet Error: %d", resp );
           break;
     }

  return buf;
}
